@@ -1,20 +1,24 @@
 boxcoxnc <-
-  function(data, method="sw", lam = seq(-3,3,0.01), plot = TRUE, alpha = 0.05, verbose = TRUE)
+  function(data, method="sw", lambda = seq(-3,3,0.01), lambda2 = NULL, plot = TRUE, alpha = 0.05, verbose = TRUE)
   {
     dname<-deparse(substitute(data))
     
     data<-as.numeric(data)
    
+    
+    if(is.null(lambda2)) lambda2<-0
+    
+    data <- data+lambda2
+
     if (is.na(min(data))==TRUE) stop("Data include NA")
-    if (min(data)<=0) stop("Data must include positive values")
-    
-    
+    if (min(data)<=0) stop("Data must include positive values. Specify shifting parameter, lambda2")
+
     
     if (method=="sw") {
       sw<-NULL
-      for (i in 1:length(lam)){
-        if (round(lam[i],2)!=0) sw<-rbind(sw,c(lam[i],shapiro.test((data**(lam[i])-1)/(lam[i]))$statistic))
-        if (round(lam[i],2)==0) sw<-rbind(sw,c(lam[i],shapiro.test(log(data))$statistic))
+      for (i in 1:length(lambda)){
+        if (round(lambda[i],2)!=0) sw<-rbind(sw,c(lambda[i],shapiro.test((data**(lambda[i])-1)/(lambda[i]))$statistic))
+        if (round(lambda[i],2)==0) sw<-rbind(sw,c(lambda[i],shapiro.test(log(data))$statistic))
       }
       
       pred.lamb<-sw[which.max(sw[,2]),1]
@@ -26,9 +30,9 @@ boxcoxnc <-
     else if (method=="ad") {
       
       ad<-NULL
-      for (i in 1:length(lam)){
-        if (round(lam[i],2)!=0) ad<-rbind(ad,c(lam[i],ad.test((data**(lam[i])-1)/(lam[i]))$statistic))
-        if (round(lam[i],2)==0) ad<-rbind(ad,c(lam[i],ad.test(log(data))$statistic))
+      for (i in 1:length(lambda)){
+        if (round(lambda[i],2)!=0) ad<-rbind(ad,c(lambda[i],ad.test((data**(lambda[i])-1)/(lambda[i]))$statistic))
+        if (round(lambda[i],2)==0) ad<-rbind(ad,c(lambda[i],ad.test(log(data))$statistic))
       }
       
       pred.lamb<-ad[which.min(ad[,2]),1]
@@ -41,9 +45,9 @@ boxcoxnc <-
       
       # Cramer Von-Mises - in nortest
       cvm<-NULL
-      for (i in 1:length(lam)){
-        if (round(lam[i],2)!=0) cvm<-rbind(cvm,c(lam[i],cvm.test((data**(lam[i])-1)/(lam[i]))$statistic))
-        if (round(lam[i],2)==0) cvm<-rbind(cvm,c(lam[i],cvm.test(log(data))$statistic))
+      for (i in 1:length(lambda)){
+        if (round(lambda[i],2)!=0) cvm<-rbind(cvm,c(lambda[i],cvm.test((data**(lambda[i])-1)/(lambda[i]))$statistic))
+        if (round(lambda[i],2)==0) cvm<-rbind(cvm,c(lambda[i],cvm.test(log(data))$statistic))
       }
       
       pred.lamb<-cvm[which.min(cvm[,2]),1]
@@ -55,9 +59,9 @@ boxcoxnc <-
     else if (method=="pt") {
       
       pt<-NULL
-      for (i in 1:length(lam)){
-        if (round(lam[i],2)!=0) pt<-rbind(pt,c(lam[i],pearson.test((data**(lam[i])-1)/(lam[i]))$statistic))
-        if (round(lam[i],2)==0) pt<-rbind(pt,c(lam[i],pearson.test(log(data))$statistic))
+      for (i in 1:length(lambda)){
+        if (round(lambda[i],2)!=0) pt<-rbind(pt,c(lambda[i],pearson.test((data**(lambda[i])-1)/(lambda[i]))$statistic))
+        if (round(lambda[i],2)==0) pt<-rbind(pt,c(lambda[i],pearson.test(log(data))$statistic))
       }
       
       pred.lamb<-pt[which.min(pt[,2]),1]
@@ -68,9 +72,9 @@ boxcoxnc <-
     else if (method=="sf") {
       
       sf<-NULL
-      for (i in 1:length(lam)){
-        if (round(lam[i],2)!=0) sf<-rbind(sf,c(lam[i],sf.test((data**(lam[i])-1)/(lam[i]))$statistic))
-        if (round(lam[i],2)==0) sf<-rbind(sf,c(lam[i],sf.test(log(data))$statistic))
+      for (i in 1:length(lambda)){
+        if (round(lambda[i],2)!=0) sf<-rbind(sf,c(lambda[i],sf.test((data**(lambda[i])-1)/(lambda[i]))$statistic))
+        if (round(lambda[i],2)==0) sf<-rbind(sf,c(lambda[i],sf.test(log(data))$statistic))
       }
       
       pred.lamb<-sf[which.max(sf[,2]),1]
@@ -82,9 +86,9 @@ boxcoxnc <-
     else if (method=="lt") {
       
       lt<-NULL
-      for (i in 1:length(lam)){
-        if (round(lam[i],2)!=0) lt<-rbind(lt,c(lam[i],lillie.test((data**(lam[i])-1)/(lam[i]))$statistic))
-        if (round(lam[i],2)==0) lt<-rbind(lt,c(lam[i],lillie.test(log(data))$statistic))
+      for (i in 1:length(lambda)){
+        if (round(lambda[i],2)!=0) lt<-rbind(lt,c(lambda[i],lillie.test((data**(lambda[i])-1)/(lambda[i]))$statistic))
+        if (round(lambda[i],2)==0) lt<-rbind(lt,c(lambda[i],lillie.test(log(data))$statistic))
       }
       
       pred.lamb<-lt[which.min(lt[,2]),1]
@@ -96,9 +100,9 @@ boxcoxnc <-
     else if (method=="jb") {
       
       jb<-NULL
-      for (i in 1:length(lam)){
-        if (round(lam[i],2)!=0) jb<-rbind(jb,c(lam[i],jarque.bera.test((data**(lam[i])-1)/(lam[i]))$statistic))
-        if (round(lam[i],2)==0) jb<-rbind(jb,c(lam[i],jarque.bera.test(log(data))$statistic))
+      for (i in 1:length(lambda)){
+        if (round(lambda[i],2)!=0) jb<-rbind(jb,c(lambda[i],jarque.bera.test((data**(lambda[i])-1)/(lambda[i]))$statistic))
+        if (round(lambda[i],2)==0) jb<-rbind(jb,c(lambda[i],jarque.bera.test(log(data))$statistic))
       }
       
       
@@ -116,7 +120,7 @@ boxcoxnc <-
         
         ac<-rnorm(length(data),0,100)
         lm1<-glm(data~ac,family=gaussian)
-        bc1<-boxcox(lm1,lam,plotit=FALSE)
+        bc1<-boxcox(lm1,lambda,plotit=FALSE)
         aclam<-bc1$x[which.max(bc1$y)]
         aclam1<-cbind(aclam1,aclam)
       }
@@ -130,8 +134,8 @@ boxcoxnc <-
     }
     
 
-    if (pred.lamb==max(lam)) stop("Enlarge the range of the lambda in a positive direction")
-    if (pred.lamb==min(lam)) stop("Enlarge the range of the lambda in a negative direction")
+    if (pred.lamb==max(lambda)) stop("Enlarge the range of the lambda in a positive direction")
+    if (pred.lamb==min(lambda)) stop("Enlarge the range of the lambda in a negative direction")
       
 
 
@@ -211,21 +215,22 @@ boxcoxnc <-
       
       if (verbose){
         cat("\n"," Box-Cox power transformation", "\n", sep = " ")
-        cat("--------------------------------------------------------", "\n", sep = " ")
+        cat("-------------------------------------------------------------------", "\n", sep = " ")
         cat("  data :", dname, "\n\n", sep = " ")
         cat("  lambda.hat :", pred.lamb, "\n\n", sep = " ")
-	cat("\n", "",nortest.name,"for transformed data", "\n", sep = " ")
-        cat("--------------------------------------------------------", "\n\n", sep = " ")
+	cat("\n", "  ",nortest.name," for transformed data ", "(alpha = ",alpha,")", "\n", sep = "")
+        cat("-------------------------------------------------------------------", "\n\n", sep = " ")
         cat("  statistic  :", statistic, "\n", sep = " ")
         cat("  p.value    :", pvalue, "\n\n", sep = " ")
         cat(if(pvalue > alpha){"  Result     : Transformed data are normal."}
             else {"  Result     : Transformed data are not normal."},"\n")
-        cat("--------------------------------------------------------", "\n\n", sep = " ")
+        cat("-------------------------------------------------------------------", "\n\n", sep = " ")
       }
       
       out<-list()
       out$method=method.name
       out$lambda.hat<-as.numeric(pred.lamb)
+      out$lambda2<-as.numeric(lambda2)
       out$statistic<-as.numeric(statistic)
       out$p.value<-as.numeric(pvalue)
       out$alpha<-as.numeric(alpha)
