@@ -7,7 +7,9 @@ confInt.boxcoxnc<- function(x, level = 0.95, verbose = TRUE,...){
 
 
 if ((level<=0)|(level>=1)) stop("Confidence level must be between 0 and 1")
-if (x$p.value<=x$alpha) stop("Transformed data are not normal")
+if (x$p.value<=x$alpha) stop(paste("Transformed data must be normally distributed at alpha = ",x$alpha,sep = ""))
+
+
 
 if (x$p.value>x$alpha){
 
@@ -21,7 +23,8 @@ if (x$lambda.hat == 0) vecbt <- exp(vectf)
 
 }
 
-vecbt<-matrix(vecbt,1,3)
+vecbt<- vecbt-x$lambda2
+vecbt<- matrix(vecbt,1,3)
 colnames(vecbt)<-c("Mean", paste((1-level)/2*100, "%",sep = ""), paste((1-(1-level)/2)*100, "%",sep = ""))
 rownames(vecbt)<-x$var.name
 
